@@ -1,34 +1,45 @@
-/***************************************************************************
-                          kgrcanvas.cpp  -  description
-                             -------------------
-    begin                : Wed Jan 23 2002
-    Copyright 2002 Marco Krüger <grisuji@gmx.de>
-    Copyright 2002 Ian Wadham <ianw2@optusnet.com.au>
-***************************************************************************/
-
-/***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/****************************************************************************
+ *    Copyright 2002  Marco Krüger <grisuji@gmx.de>                         *
+ *    Copyright 2002  Ian Wadham <iandw.au@gmail.com>                       *
+ *    Copyright 2006  Mauricio Piacentini <mauricio@tabuleiro.com>          *
+ *    Copyright 2007  Mauricio Piacentini <mauricio@tabuleiro.com>          *
+ *    Copyright 2007  Luciano Montanaro <mikelima@cirulla.net>              *
+ *    Copyright 2008  Luciano Montanaro <mikelima@cirulla.net>              *
+ *    Copyright 2009  Ian Wadham <iandw.au@gmail.com>                       *
+ *                                                                          *
+ *    This program is free software; you can redistribute it and/or         *
+ *    modify it under the terms of the GNU General Public License as        *
+ *    published by the Free Software Foundation; either version 2 of        *
+ *    the License, or (at your option) any later version.                   *
+ *                                                                          *
+ *    This program is distributed in the hope that it will be useful,       *
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *    GNU General Public License for more details.                          *
+ *                                                                          *
+ *    You should have received a copy of the GNU General Public License     *
+ *    along with this program.  If not, see <http://www.gnu.org/licenses/>. *
+ ****************************************************************************/
 
 #include "kgrcanvas.h"
-#include "klocale.h"
+#include "kgrplayfield.h"
+#include "kgrsprite.h"
 
-#include <QPixmap>
-#include <QList>
 #include <QLabel>
+#include <QCursor>
 #include <QMouseEvent>
 
 #include <KDebug>
-
 #include <KConfig>
+#include <KLocale>
 
 #include <cmath>
 
+const StartFrame animationStartFrames [nAnimationTypes] = {
+                 RIGHTWALK1, LEFTWALK1, RIGHTCLIMB1, LEFTCLIMB1,
+                 CLIMB1, CLIMB1, FALL1, FALL2,
+                 (StartFrame) 0,	// Calculate OPEN_BRICK frame later.
+                 (StartFrame) 0};	// Calculate CLOSE_BRICK frame later.
 
 // Helper function: find how many tiles are needed to cover at least w pixels
 static int numSections(int w, int sectionWidth)
@@ -46,21 +57,26 @@ static QString livesText(int lives)
     return ki18n ("Lives: ").toString() + QString::number(lives).rightJustified(3, '0');
 }
 
-KGrCanvas::KGrCanvas (QWidget * parent, const double scale,
-                        const QString & systemDataDir)
-                        : KGameCanvasWidget (parent),
+KGrCanvas::KGrCanvas (QWidget * parent, const double scale)
+                          :
+                          KGameCanvasWidget (parent),
                           firstSceneDrawn (false),
+                          m_replayText(0),
                           topLeft (0, 0), bgw (4 * STEP), bgh (4 * STEP),
                           m_scoreText(0),
                           m_livesText(0),
                           m_scoreDisplay(0),
                           m_livesDisplay(0),
                           m_fadingTimeLine (1000, this),
-                          theme (systemDataDir)
-
+                          emptySprites (0),
+                          theme(),
+                          heroId (0)
 {
     resizeCount = 0;		// IDW
 
+    // Workaround for bug 197686 (lost Esc shortcut after hovering over menu).
+    setFocusPolicy (Qt::WheelFocus);
+
     kDebug() << "Called KGrCanvas::KGrCanvas ..." << this->size();
     m = new QCursor();		// For handling the mouse.
 
@@ -70,7 +86,7 @@ KGrCanvas::KGrCanvas (QWidget * parent, const double scale,
 
     scaleStep = STEP;		// Initial scale is 1:1.
     baseScale = scaleStep;
-    baseFontSize = fontInfo().pointSize() + 2;
+    // IDW baseFontSize = fontInfo().pointSize() + 2;
     scaleStep = (int) ((scale * STEP) + 0.05);
     kDebug() << "Scale" << scale << "Scaled Step" << scaleStep;
 
@@ -79,10 +95,8 @@ KGrCanvas::KGrCanvas (QWidget * parent, const double scale,
     border = 4;			// Make border at least tiles wide all around.
     lineDivider = 8;		// Make lines of inner border 1/8 tile wide.
 
-    heroSprite = 0;
-
-    // Create an empty list of enemy sprites.
-    enemySprites = new QList<KGrSprite *>();
+    // Create an empty list of sprites.
+    sprites = new QList<KGrSprite *>();
 
     kDebug() << "Calling initView() ...";
     initView();			// Set up the graphics, etc.
@@ -110,8 +124,8 @@ KGrCanvas::KGrCanvas (QWidget * parent, const double scale,
     m_spotLight = new KGameCanvasPicture (this);
     m_fadingTimeLine.setCurveShape (QTimeLine::LinearCurve);
     m_fadingTimeLine.setUpdateInterval (60);
-    connect (&m_fadingTimeLine, SIGNAL (valueChanged (qreal)),
-                this, SLOT (drawSpotLight (qreal)));
+    connect (&m_fadingTimeLine, SIGNAL (valueChanged(qreal)),
+                this, SLOT (drawSpotLight(qreal)));
     connect (&m_fadingTimeLine, SIGNAL (finished()),
                 this, SIGNAL (fadeFinished()));
 }
@@ -121,18 +135,16 @@ KGrCanvas::~KGrCanvas()
     tileset->clear();
     heroFrames->clear();
     enemyFrames->clear();
-    deleteEnemySprites();
-    while (!borderRectangles.isEmpty())
-	delete borderRectangles.takeFirst();
-    while (!borderElements.isEmpty())
-	delete borderElements.takeFirst();
+    deleteAllSprites();
+
+    qDeleteAll(borderRectangles);
+    qDeleteAll(borderElements);
 
     delete playfield;
     delete tileset;
     delete heroFrames;
     delete enemyFrames;
-    delete heroSprite;
-    delete enemySprites;
+    delete sprites;
     delete m_spotLight;
     delete m_scoreText;
     delete m_livesText;
@@ -177,7 +189,7 @@ void KGrCanvas::drawTheScene (bool changePixmaps)
 
     // Draw the tiles and background in the playfield.
     if (playfield) {
-	loadBackground();
+	loadBackground (-1);		// Load/render/cache all backgrounds.
 
 	if (changePixmaps) {
 	    tileset->clear();
@@ -214,32 +226,33 @@ void KGrCanvas::drawTheScene (bool changePixmaps)
     }
     kDebug() << t.restart() << "msec.  Hero + enemies done.";
 
-    int spriteframe;
+    int    spriteframe;
     QPoint spriteloc;
+    char   spritetype;
 
-    // Draw the hero.
-    if (heroSprite) {
-        spriteframe = heroSprite->currentFrame();
-        spriteloc = heroSprite->currentLoc();
-        heroSprite->addFrames (heroFrames, topLeft, scale);
+    if (sprites) {
+        foreach (KGrSprite * sprite, (* sprites)) {
+            if (sprite) {
+                spriteframe = sprite->currentFrame();
+                spriteloc   = sprite->currentLoc();
+                spritetype  = sprite->spriteType();
 
-        // Force a re-draw of both pixmap and position.
-        heroSprite->move (0, 0, (spriteframe > 0) ? 0 : 1);
-        heroSprite->move (spriteloc.x(), spriteloc.y(), spriteframe);
-    }
-
-    if (enemySprites) {
-        for (int i = 0; i < enemySprites->size(); ++i) {
-            // kDebug() << "accessing enemySprite" << i;
-            KGrSprite * thisenemy = enemySprites->at (i);
-            if (thisenemy) {
-                spriteframe = thisenemy->currentFrame();
-                spriteloc = thisenemy->currentLoc();
-                thisenemy->addFrames (enemyFrames, topLeft, scale);
+                switch (spritetype) {
+                case HERO:
+                    sprite->addFrames (heroFrames,  topLeft, scale);
+                    break;
+                case ENEMY:
+                    sprite->addFrames (enemyFrames, topLeft, scale);
+                    break;
+                case BRICK:
+                    sprite->addFrames (tileset, topLeft, scale);
+                    break;
+                }
 
                 // Force re-draw of both pixmap and position.
-                thisenemy->move (0, 0, (spriteframe > 0) ? 0 : 1);
-                thisenemy->move (spriteloc.x(), spriteloc.y(), spriteframe);
+                sprite->move ((spriteloc.x() > 0) ? 0 : 1, 0,
+                              (spriteframe > 0) ? (spriteframe - 1) : 1);
+                sprite->move (spriteloc.x(), spriteloc.y(), spriteframe);
             }
         }
     }
@@ -253,14 +266,19 @@ void KGrCanvas::drawTheScene (bool changePixmaps)
     makeTitle();
     setTitle (t);
 
+    // Do the same for the replay message.
+    makeReplayMessage();
+
     // Create and position the score and lives text areas
     QFont f;
-    f.setPixelSize (imgH - 2);
+    f.setPixelSize ((imgH * 2) / 3);		// Set font 2/3 of cell height.
     f.setWeight (QFont::Bold);
     f.setStretch (QFont::Expanded);
     m_scoreText->setFont (f);
     m_scoreText->setColor (theme.textColor());
-    int vTextPos = (1 + imgH) * nCellsH + 1;
+
+    // Center the text in the second cell below the play area (below border).
+    int vTextPos = (nCellsH + 1) * imgH + (imgH / 6);
     QPoint scorePos = topLeft + QPoint (imgW, vTextPos);
 
     m_livesText->setFont (f);
@@ -326,7 +344,7 @@ void KGrCanvas::resizeEvent (QResizeEvent * event)
     
     double w = (double) event->size().width()  / (nCellsW + border);
     double h = (double) event->size().height() / (nCellsH + border);
-    int cellSize = (w < h) ? (int) (w + 0.05) : (int) (h + 0.05);
+    int cellSize = (int) (qMin(w, h) + 0.05);
 
     imgW = cellSize;
     imgH = cellSize;
@@ -352,7 +370,7 @@ KGrTheme::TileType KGrCanvas::tileForType(char type)
     switch (type) {
     case NUGGET:
 	return KGrTheme::GoldTile;
-    case POLE:
+    case BAR:
 	return KGrTheme::BarTile;
     case LADDER:
 	return KGrTheme::LadderTile;
@@ -362,7 +380,7 @@ KGrTheme::TileType KGrCanvas::tileForType(char type)
 	return KGrTheme::HeroTile;
     case ENEMY:
 	return KGrTheme::EnemyTile;
-    case BETON:
+    case CONCRETE:
 	return KGrTheme::ConcreteTile;
     case BRICK:
 	return KGrTheme::BrickTile;
@@ -400,11 +418,11 @@ int KGrCanvas::tileNumber (KGrTheme::TileType type, int x, int y)
     }
 }
 
-void KGrCanvas::paintCell (int x, int y, char type, int offset)
+void KGrCanvas::paintCell (const int i, const int j, const char type, const int offset)
 {
     KGrTheme::TileType tileType = tileForType (type);
     // In KGrGame, the top-left visible cell is [1,1]: in KGrPlayfield [0,0].
-    x--; y--;
+    int x = i - 1, y = j - 1;
     switch (offset) {
     case 1: tileType = KGrTheme::BrickAnimation1Tile; break;
     case 2: tileType = KGrTheme::BrickAnimation2Tile; break;
@@ -444,20 +462,24 @@ void KGrCanvas::setTitle (const QString &newTitle)
 
 void KGrCanvas::makeTitle()
 {
+    // TODO - Used to use baseScale, baseFontSize and scaleStep: obsolete now?
     if (title != 0)
         title->close();			// Close and delete previous title.
 
+    QFont f (fontInfo().family(), 12, QFont::Bold);
+    f.setPixelSize ((imgH * 2) / 3);	// Set font 2/3 of cell height.
     title = new QLabel ("", this);
+
+    // Use the top half of the space above the play area (above the border).
     int lw = imgW / lineDivider;	// Line width (as used in makeBorder()).
-    title->resize (width(), topLeft.y() - lw);
+    title->resize (width(), (topLeft.y() - lw) / 2);
     title->move (0, 0);
     QPalette palette;
     palette.setColor (title->backgroundRole(), theme.borderColor());
     palette.setColor (title->foregroundRole(), theme.textColor());
     title->setPalette (palette);
-    title->setFont (QFont (fontInfo().family(),
-                 (baseFontSize * scaleStep) / baseScale, QFont::Bold));
-    title->setAlignment (Qt::AlignCenter);
+    title->setFont (f);
+    title->setAlignment (Qt::AlignBottom | Qt::AlignHCenter);
     title->setAttribute (Qt::WA_QuitOnClose, false); //Otherwise the close above might exit app
     title->raise();
     title->show();
@@ -476,8 +498,50 @@ void KGrCanvas::updateLives (int lives)
         m_livesText->setText (livesText(lives));
 }
 
+void KGrCanvas::makeReplayMessage()
+{
+    bool wasVisible = false;
+    if (m_replayText != 0) {
+        wasVisible = m_replayText->isVisible();
+        m_replayText->close();		// Close and delete previous message.
+    }
+
+    QFont f (fontInfo().family(), 12, QFont::Bold);
+    f.setPixelSize ((imgH * 3) / 5);	// Set font 60% of cell height.
+    m_replayText = new QLabel (i18n("Click anywhere to begin live play"), this);
+
+    // Use the lower half of the space above the play area (above the border).
+    int lw = imgW / lineDivider;	// Line width (as used in makeBorder()).
+    m_replayText->resize (width(), (topLeft.y() - lw) / 2);
+    m_replayText->move (0, (topLeft.y() - lw) / 2);
+    QPalette palette;
+    palette.setColor (m_replayText->backgroundRole(), theme.borderColor());
+    palette.setColor (m_replayText->foregroundRole(), theme.textColor());
+    m_replayText->setPalette (palette);
+    m_replayText->setFont (f);
+    // m_replayText->setAlignment (Qt::AlignBottom | Qt::AlignHCenter);
+    m_replayText->setAlignment (Qt::AlignVCenter | Qt::AlignHCenter);
+    m_replayText->setAttribute (Qt::WA_QuitOnClose, false); //Otherwise the close above might exit app
+    if (wasVisible) {
+        m_replayText->raise();
+        m_replayText->show();
+    }
+}
+
+void KGrCanvas::showReplayMessage (bool onOff)
+{
+    if (onOff) {
+        m_replayText->raise();
+        m_replayText->show();
+    }
+    else {
+        m_replayText->hide();
+    }
+}
+
 void KGrCanvas::mousePressEvent (QMouseEvent * mouseEvent)
 {
+    // kDebug() << "Button" << mouseEvent->button();
     emit mouseClick (mouseEvent->button());
 }
 
@@ -486,82 +550,193 @@ void KGrCanvas::mouseReleaseEvent (QMouseEvent * mouseEvent)
     emit mouseLetGo (mouseEvent->button());
 }
 
-QPoint KGrCanvas::getMousePos()
+void KGrCanvas::getMousePos (int & i, int & j)
 {
-    int i, j;
     QPoint p = mapFromGlobal (m->pos());
 
-    // In KGoldrunner, the top-left visible cell is [1,1]: in KGrSprite [0,0].
+    // In KGoldrunner, the top-left visible cell is [1,1].
     i = ((p.x() - topLeft.x()) / imgW) + 1;
     j = ((p.y() - topLeft.y()) / imgH) + 1;
 
-    return (QPoint (i, j));
+    // Make sure i and j are within the KGoldrunner grid.
+    i = (i < 1) ? 1 : ((i > nCellsW) ? nCellsW : i);
+    j = (j < 1) ? 1 : ((j > nCellsH) ? nCellsH : j);
 }
 
-void KGrCanvas::setMousePos (int i, int j)
+void KGrCanvas::setMousePos (const int i, const int j)
 {
     // In KGoldrunner, the top-left visible cell is [1,1]: in KGrSprite [0,0].
-    i--; j--;
-    m->setPos (mapToGlobal (QPoint (
-                                topLeft.x() + i * imgW + imgW / 2,
-                                topLeft.y() + j * imgH + imgH / 2)));
+    m->setPos (mapToGlobal (QPoint (topLeft.x() + (i - 1) * imgW + imgW / 2,
+                                    topLeft.y() + (j - 1) * imgH + imgH / 2)));
 }
 
-void KGrCanvas::makeHeroSprite (int i, int j, int startFrame)
+void KGrCanvas::animate (bool missed)
 {
-    heroSprite = new KGrSprite (this);
+    foreach (KGrSprite * sprite, (* sprites)) {
+        if (sprite != 0) {
+            sprite->animate (missed);
+        }
+    }
+}
 
+int KGrCanvas::makeSprite (const char type, int i, int j)
+{
+    int spriteId;
+    KGrSprite * sprite = new KGrSprite (this, type, TickTime);
+
+    if ((emptySprites > 0) && ((spriteId = sprites->lastIndexOf (0)) >= 0)) {
+        // Re-use a slot previously occupied by a transient member of the list.
+        (* sprites) [spriteId] = sprite;
+        emptySprites--;
+    }
+    else {
+        // Otherwise, add to the end of the list.
+        spriteId = sprites->count();
+        sprites->append (sprite);
+        emptySprites = 0;
+    }
+
+    int frame1   = animationStartFrames [FALL_L];
     double scale = (double) imgW / (double) bgw;
-    heroSprite->addFrames (heroFrames, topLeft, scale);
 
-    // In KGoldrunner, the top-left visible cell is [1,1]: in KGrSprite [0,0].
-    i--; j--;
-    heroSprite->move (i * bgw, j * bgh, startFrame);
-    heroSprite->setZ (1);
-    heroSprite->setVisible (true);
-}
+    switch (type) {
+    case HERO:
+        heroId = spriteId;
+        sprite->addFrames (heroFrames, topLeft, scale);
+        sprite->setZ (1);
+        break;
+    case ENEMY:
+        sprite->addFrames (enemyFrames, topLeft, scale);
+        sprite->setZ (2);
+        break;
+    case BRICK:
+        sprite->addFrames (tileset, topLeft, scale);
+        frame1 = KGrTheme::BrickAnimation1Tile;
 
-void KGrCanvas::setHeroVisible (bool newState)
-{
-    heroSprite->setVisible (newState);		// Show or hide the hero.
-}
+        // The hero and enemies must be painted in front of dug bricks.
+        sprite->stackUnder (sprites->at (heroId));
 
-void KGrCanvas::makeEnemySprite (int i, int j, int startFrame)
-{
-    KGrSprite * enemySprite = new KGrSprite (this);
-
-    double scale = (double) imgW / (double) bgw;
-    enemySprite->addFrames (enemyFrames, topLeft, scale);
-    enemySprites->append (enemySprite);
-
-    // In KGoldrunner, the top-left visible cell is [1,1]: in KGrSprite [0,0].
-    i--; j--;
-    enemySprite->move (i * bgw, j * bgh, startFrame);
-    enemySprite->setZ (2);
-    enemySprite->show();
-}
-
-void KGrCanvas::moveHero (int x, int y, int frame)
-{
-    // In KGoldrunner, the top-left visible cell is [1,1]: in KGrSprite [0,0].
-    heroSprite->move (x - bgw, y - bgh, frame);
-}
-
-void KGrCanvas::moveEnemy (int id, int x, int y, int frame, int nuggets)
-{
-    if (nuggets != 0) {				// If enemy is carrying gold,
-        frame = frame + goldEnemy;		// show him with gold outline.
+        // Erase the brick-image so that animations are visible in all themes.
+        paintCell (i, j, FREE, 0);
+        break;
+    default:
+        break;
     }
 
     // In KGoldrunner, the top-left visible cell is [1,1]: in KGrSprite [0,0].
-    // kDebug() << "accessing enemySprite" << id;
-    enemySprites->at (id)->move (x - bgw, y - bgh, frame);
+    sprite->move ((i - 1) * bgw, (j - 1) * bgh, frame1);
+    sprite->show();
+
+    // kDebug() << "Sprite ID" << spriteId << "sprite type" << type
+             // << "at" << i << j;
+    return spriteId;
 }
 
-void KGrCanvas::deleteEnemySprites()
+void KGrCanvas::startAnimation (const int id, const bool repeating,
+                                const int i, const int j, const int time,
+                                const Direction dirn, const AnimationType type)
 {
-    while (!enemySprites->isEmpty())
-        delete enemySprites->takeFirst();
+    // TODO - Put most of this in helper code, based on theme parameters.
+    int frame = animationStartFrames [type];
+    int nFrames = 8;
+    int nFrameChanges = 4;
+    int dx = 0;
+    int dy = 0;
+
+    switch (dirn) {
+    case RIGHT:
+        dx    = +1;
+        break;
+    case LEFT:
+        dx    = -1;
+        break;
+    case DOWN:
+        dy = +1;
+        if ((type == FALL_R) || (type == FALL_L)) {
+            nFrames = 1;
+        }
+        else {
+            nFrames = 2;
+        }
+        break;
+    case UP:
+        dy = -1;
+        nFrames = 2;
+        break;
+    case STAND:
+        switch (type) {
+        case OPEN_BRICK:
+            nFrames = 5;
+            frame = tileNumber (KGrTheme::BrickAnimation1Tile, i, j);
+            break;
+        case CLOSE_BRICK:
+            nFrames = 4;
+            frame = tileNumber (KGrTheme::BrickAnimation6Tile, i, j);
+            break;
+        default:
+            // Show a standing hero or enemy, using the previous StartFrame.
+            nFrames = 0; 
+            break;
+        }
+        break;
+    default:
+        break;
+    }
+
+    // TODO - Generalise nFrameChanges = 4, also the tick time = 20 new sprite.
+    sprites->at(id)->setAnimation (repeating, (i - 1) * bgw, (j - 1) * bgh,
+                    frame, nFrames, dx * bgw, dy * bgh, time, nFrameChanges);
+}
+
+// void KGrCanvas::resynchAnimation (const int id, const int i, const int j,
+                                  // const bool stop)
+// {
+    // Write this code only if it is really needed.
+// }
+
+void KGrCanvas::gotGold (const int spriteId, const int i, const int j,
+                         const bool spriteHasGold, const bool lost)
+{
+    // Hide collected gold or show dropped gold, but not if the gold was lost.
+    if (! lost) {
+        paintCell (i, j, (spriteHasGold) ? FREE : NUGGET);
+    }
+
+    // If the rules allow, show whether or not an enemy sprite is carrying gold.
+    if (enemiesShowGold && (sprites->at(spriteId)->spriteType() == ENEMY)) {
+        sprites->at(spriteId)->setFrameOffset (spriteHasGold ? goldEnemy : 0);
+    }
+}
+
+void KGrCanvas::showHiddenLadders (const QList<int> & ladders, const int width)
+{
+    foreach (int offset, ladders) {
+        int i = offset % width;
+        int j = offset / width;
+        paintCell (i, j, LADDER);
+    }
+}
+
+void KGrCanvas::deleteSprite (const int spriteId)
+{
+    QPoint loc   = sprites->at(spriteId)->currentLoc();
+    bool   brick = (sprites->at(spriteId)->spriteType() == BRICK);
+
+    delete sprites->at(spriteId);
+    (* sprites)[spriteId] = 0;
+    emptySprites++;
+
+    if (brick) {
+        // Dug-brick sprite erased: restore the tile that was at that location.
+        paintCell ((loc.x()/bgw) + 1, (loc.y()/bgh) + 1, BRICK, 0);
+    }
+}
+
+void KGrCanvas::deleteAllSprites()
+{
+    qDeleteAll(*sprites);
+    sprites->clear();
+    emptySprites = 0;
 }
 
 QPixmap KGrCanvas::getPixmap (char type)
@@ -585,24 +760,35 @@ void KGrCanvas::initView()
     goldEnemy = 36;			// Offset of gold-carrying frames.
 }
 
-void KGrCanvas::setLevel (unsigned int l)
+void KGrCanvas::setLevel (unsigned int lev)
 {
-    if (l != level) {
-        level= l;
+    if (lev != level) {
+        level = lev;
         if (theme.backgroundCount() > 1) {
-            loadBackground();
+            loadBackground (lev);	// Load (uncache) background for level.
         }
     }
 }
 
-void KGrCanvas::loadBackground()
+void KGrCanvas::loadBackground (const int lev)
 {
-    kDebug() << "loadBackground called";
+    kDebug() << "Parameter" << lev << "level" << level;
     bool fillCanvas = !theme.isBorderRequired(); // Background must fill canvas?
     int w = fillCanvas ? (this->width())  : (nCellsW * imgW);
     int h = fillCanvas ? (this->height()) : (nCellsH * imgH);
+    QPixmap background;
     if (theme.backgroundCount() > 0) {
-        QPixmap background = theme.background (w, h, level);
+        if (lev < 0) {
+            // When initially loading a theme, make sure other backgrounds are
+            // loaded, rendered and cached, to save SVG reloads in later runs,
+            // e.g. if player quits before changing to a new level.
+            for (int i = 1; i <= theme.backgroundCount(); i++) {
+                background = theme.background (w, h, i);
+            }
+        }
+        // When initially loading a theme, use "background0" or "background",
+        // but normally use the background calculated for the current level.
+        background = theme.background (w, h, level <= 0 ? 0 : level);
         playfield->setBackground (true, background, 
                 theme.isBorderRequired() ? topLeft : QPoint (0, 0));
     }
@@ -626,10 +812,10 @@ void KGrCanvas::makeBorder ()
     int tlY = topLeft.y();
     int lw = imgW / lineDivider;		// Line width.
 
-    while (!borderRectangles.isEmpty())
-	delete borderRectangles.takeFirst();
-    while (!borderElements.isEmpty())
-	delete borderElements.takeFirst();
+    qDeleteAll(borderRectangles);
+    borderRectangles.clear();
+    qDeleteAll(borderElements);
+    borderElements.clear();
 
     // a fancy border can be specified in the SVG file; if that is unavailable,
     // a simple border can be specified in the theme properties file.
