@@ -35,7 +35,7 @@
 #include <KConfigDialog>
 #include <KFileDialog>
 #include <KGameThemeSelector>
-#include <knewstuff2/engine.h>
+#include <KNS3/DownloadDialog>
 #include <KStandardDirs>
 #include <KStandardGameAction>
 #include <KToggleAction>
@@ -46,7 +46,7 @@
 
 namespace Kigo {
 
-MainWindow::MainWindow(QWidget *parent)
+MainWindow::MainWindow(const QString &fileName, QWidget *parent)
     : KXmlGuiWindow(parent), m_game(new Game(this))
     , m_gameScene(new GameScene(m_game)), m_gameView(new GameView(m_gameScene))
 {
@@ -59,11 +59,13 @@ MainWindow::MainWindow(QWidget *parent)
 
     connect(m_game, SIGNAL(waiting(bool)), this, SLOT(showBusy(bool)));
     connect(m_game, SIGNAL(consecutivePassMovesPlayed(int)), this, SLOT(showFinishGameAction()));
-    connect(m_game, SIGNAL(resigned(const Player &)), this, SLOT(finishGame()));
-    connect(m_game, SIGNAL(passMovePlayed(const Player &)), this, SLOT(passMovePlayed(const Player &)));
+    connect(m_game, SIGNAL(resigned(Player)), this, SLOT(finishGame()));
+    connect(m_game, SIGNAL(passMovePlayed(Player)), this, SLOT(passMovePlayed(Player)));
 
     if (isBackendWorking()) {
-        newGame();
+        if (!loadGame(fileName)) {
+            newGame();
+        }
     } else {
         backendError();
     }
@@ -90,7 +92,7 @@ void MainWindow::newGame()
 
     disconnect(m_game, SIGNAL(canRedoChanged(bool)), m_redoMoveAction, SLOT(setEnabled(bool)));
     disconnect(m_game, SIGNAL(canUndoChanged(bool)), m_undoMoveAction, SLOT(setEnabled(bool)));
-    disconnect(m_game, SIGNAL(currentPlayerChanged(const Player &)), this, SLOT(playerChanged()));
+    disconnect(m_game, SIGNAL(currentPlayerChanged(Player)), this, SLOT(playerChanged()));
 
     m_gameDock->setVisible(false);
     m_gameDock->toggleViewAction()->setEnabled(false);
@@ -107,6 +109,13 @@ void MainWindow::loadGame()
 {
     QString fileName = KFileDialog::getOpenFileName(KUrl(KStandardDirs::locate("appdata", "games/")), "*.sgf");
     if (!fileName.isEmpty()) {
+        loadGame(fileName);
+    }
+}
+
+bool MainWindow::loadGame(const QString &fileName)
+{
+    if (!fileName.isEmpty() && QFile(fileName).exists()) {
         m_game->start(Preferences::engineCommand());
 
         m_gameScene->showTerritory(false);
@@ -126,7 +135,7 @@ void MainWindow::loadGame()
 
         disconnect(m_game, SIGNAL(canRedoChanged(bool)), m_redoMoveAction, SLOT(setEnabled(bool)));
         disconnect(m_game, SIGNAL(canUndoChanged(bool)), m_undoMoveAction, SLOT(setEnabled(bool)));
-        disconnect(m_game, SIGNAL(currentPlayerChanged(const Player &)), this, SLOT(playerChanged()));
+        disconnect(m_game, SIGNAL(currentPlayerChanged(Player)), this, SLOT(playerChanged()));
 
         m_gameDock->setVisible(false);
         m_gameDock->toggleViewAction()->setEnabled(false);
@@ -137,23 +146,24 @@ void MainWindow::loadGame()
 
         m_setupWidget->loadedGame(fileName);
         m_gameScene->showMessage(i18n("Set up a loaded game..."));
+        return true;
     } else {
         m_gameScene->showMessage(i18n("Unable to load game..."));
         //Note: New game implied here
+        return false;
     }
 }
 
 void MainWindow::getMoreGames()
 {
-    KNS::Engine engine(0);
+    KNS3::DownloadDialog dialog("kigo-games.knsrc", this);
+    dialog.exec();
 
-    if (engine.init("kigo-games.knsrc")) {
-        KNS::Entry::List entries = engine.downloadDialogModal(this);
-        /*if (entries.size() > 0) {
-            // do something with the modified entries here if you want
-            // such as rescaning your data folder or whatnot
-        }*/
-    }
+    /*KNS3::Entry::List entries = dialog.changedEntries();
+    if (entries.size() > 0) {
+        // do something with the modified entries here if you want
+        // such as rescaning your data folder or whatnot
+    }*/
 }
 
 void MainWindow::backendError()
@@ -174,7 +184,7 @@ void MainWindow::backendError()
 
     disconnect(m_game, SIGNAL(canRedoChanged(bool)), m_redoMoveAction, SLOT(setEnabled(bool)));
     disconnect(m_game, SIGNAL(canUndoChanged(bool)), m_undoMoveAction, SLOT(setEnabled(bool)));
-    disconnect(m_game, SIGNAL(currentPlayerChanged(const Player &)), this, SLOT(playerChanged()));
+    disconnect(m_game, SIGNAL(currentPlayerChanged(Player)), this, SLOT(playerChanged()));
     m_gameDock->setVisible(false);
     m_gameDock->toggleViewAction()->setEnabled(false);
     m_movesDock->setVisible(false);
@@ -214,6 +224,8 @@ void MainWindow::startGame()
 
         m_gameView->setInteractive(true);
         m_undoView->setEnabled(true);
+
+        m_gameScene->showPlacementMarker(true);
     } else {
         m_passMoveAction->setEnabled(false);
         m_hintAction->setEnabled(false);
@@ -221,9 +233,11 @@ void MainWindow::startGame()
 
         m_gameView->setInteractive(false);
         m_undoView->setEnabled(false);
+
+        m_gameScene->showPlacementMarker(false);
     }
 
-    connect(m_game, SIGNAL(currentPlayerChanged(const Player &)), this, SLOT(playerChanged()));
+    connect(m_game, SIGNAL(currentPlayerChanged(Player)), this, SLOT(playerChanged()));
     // Trigger the slot once to make a move if the starting player
     // (black) is a computer player.
     playerChanged();
@@ -257,7 +271,7 @@ void MainWindow::finishGame()
 
     m_undoView->setEnabled(false);
 
-    kDebug() << "Fetching final score from engine ...";
+    // kDebug() << "Fetching final score from engine ...";
     Score score = m_game->estimatedScore();
     QString name;
     if (score.color() == 'W') {
@@ -267,7 +281,7 @@ void MainWindow::finishGame()
     }
     // Show a score message that stays visible until the next
     // popup message arrives
-    m_gameScene->showMessage(i18n("%1 has won this game with a score of %2 (bounded by %3 and %4).",
+    m_gameScene->showMessage(i18n("%1 won with a score of %2 (bounds: %3 and %4).",
                              name, score.score(), score.upperBound(), score.lowerBound()), 0);
 }
 
@@ -308,15 +322,15 @@ void MainWindow::showPreferences()
 
     KConfigDialog *dialog = new KConfigDialog(this, "settings", Preferences::self());
     dialog->addPage(new GeneralConfig(), i18n("General"), "preferences-other");
-    dialog->addPage(new KGameThemeSelector(dialog, Preferences::self()), i18n("Themes"), "games-config-theme");
+    dialog->addPage(new KGameThemeSelector(dialog, Preferences::self(), KGameThemeSelector::NewStuffDisableDownload), i18n("Themes"), "games-config-theme");
     dialog->setHelp(QString(), "Kigo");
-    connect(dialog, SIGNAL(settingsChanged(const QString &)), this, SLOT(applyPreferences()));
+    connect(dialog, SIGNAL(settingsChanged(QString)), this, SLOT(applyPreferences()));
     dialog->show();
 }
 
 void MainWindow::applyPreferences()
 {
-    //kDebug() << "Update settings based on changed configuration...";
+    // kDebug() << "Update settings based on changed configuration...";
     m_gameScene->showLabels(Preferences::showBoardLabels());
 
     ThemeRenderer::self()->loadTheme(Preferences::theme());
@@ -388,25 +402,25 @@ void MainWindow::setupActions()
     // Game menu
     m_newGameAction = KStandardGameAction::gameNew(this, SLOT(newGame()), actionCollection());
     m_loadGameAction = KStandardGameAction::load(this, SLOT(loadGame()), actionCollection());
-    m_getMoreGamesAction = new KAction(KIcon("get-hot-new-stuff"), i18nc("@action", "Get More Games..."), this);
+    m_getMoreGamesAction = new KAction(KIcon( QLatin1String( "get-hot-new-stuff") ), i18nc("@action", "Get More Games..." ), this);
     m_getMoreGamesAction->setShortcut(Qt::CTRL + Qt::Key_G);
     m_getMoreGamesAction->setToolTip(i18nc("@action", "Get More Games..."));
     connect(m_getMoreGamesAction, SIGNAL(triggered(bool)), this, SLOT(getMoreGames()));
-    actionCollection()->addAction("get_more_games", m_getMoreGamesAction);
+    actionCollection()->addAction( QLatin1String( "get_more_games" ), m_getMoreGamesAction);
     m_saveAction = KStandardGameAction::save(this, SLOT(saveGame()), actionCollection());
     KStandardGameAction::quit(this, SLOT(close()), actionCollection());
 
-    m_startGameAction = new KAction(KIcon("media-playback-start"), i18nc("@action", "Start Game"), this);
+    m_startGameAction = new KAction(KIcon( QLatin1String( "media-playback-start") ), i18nc("@action", "Start Game" ), this);
     m_startGameAction->setShortcut(Qt::Key_S);
     m_startGameAction->setToolTip(i18nc("@action", "Start Game"));
     connect(m_startGameAction, SIGNAL(triggered(bool)), this, SLOT(startGame()));
-    actionCollection()->addAction("game_start", m_startGameAction);
+    actionCollection()->addAction( QLatin1String( "game_start" ), m_startGameAction);
 
-    m_finishGameAction = new KAction(KIcon("media-playback-stop"), i18nc("@action", "Finish Game"), this);
+    m_finishGameAction = new KAction(KIcon( QLatin1String( "media-playback-stop") ), i18nc("@action", "Finish Game" ), this);
     m_finishGameAction->setShortcut(Qt::Key_F);
     m_finishGameAction->setToolTip(i18nc("@action", "Finish Game"));
     connect(m_finishGameAction, SIGNAL(triggered(bool)), this, SLOT(finishGame()));
-    actionCollection()->addAction("game_finish", m_finishGameAction);
+    actionCollection()->addAction( QLatin1String( "game_finish" ), m_finishGameAction);
 
     // Move menu
     m_undoMoveAction = KStandardGameAction::undo(this, SLOT(undo()), actionCollection());
@@ -417,11 +431,11 @@ void MainWindow::setupActions()
     m_hintAction = KStandardGameAction::hint(this, SLOT(hint()), actionCollection());
 
     // View menu
-    m_moveNumbersAction = new KToggleAction(KIcon("lastmoves"), i18nc("@action:inmenu View", "Show Move &Numbers"), this);
+    m_moveNumbersAction = new KToggleAction(KIcon( QLatin1String( "lastmoves") ), i18nc("@action:inmenu View", "Show Move &Numbers" ), this);
     m_moveNumbersAction->setShortcut(Qt::Key_N);
     m_moveNumbersAction->setChecked(Preferences::showMoveNumbers());
     connect(m_moveNumbersAction, SIGNAL(toggled(bool)), m_gameScene, SLOT(showMoveNumbers(bool)));
-    actionCollection()->addAction("move_numbers", m_moveNumbersAction);
+    actionCollection()->addAction( QLatin1String( "move_numbers" ), m_moveNumbersAction);
 
     // Settings menu
     KStandardAction::preferences(this, SLOT(showPreferences()), actionCollection());
@@ -431,32 +445,31 @@ void MainWindow::setupDockWindows()
 {
     // Setup dock
     m_setupDock = new QDockWidget(i18nc("@title:window", "Game Setup"), this);
-    m_setupDock->setObjectName("setupDock");
+    m_setupDock->setObjectName( QLatin1String("setupDock" ));
     m_setupDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
     m_setupWidget = new SetupWidget(m_game, this);
     m_setupDock->setWidget(m_setupWidget);
     connect(m_setupWidget, SIGNAL(startClicked()), this, SLOT(startGame()));
     //m_setupDock->toggleViewAction()->setText(i18nc("@title:window", "Game setup"));
     //m_setupDock->toggleViewAction()->setShortcut(Qt::Key_S);
-    //actionCollection()->addAction("show_setup_panel", m_setupDock->toggleViewAction());
+    //actionCollection()->addAction( QLatin1String( "show_setup_panel" ), m_setupDock->toggleViewAction());
     addDockWidget(Qt::BottomDockWidgetArea, m_setupDock);
 
     // Game dock
     m_gameDock = new QDockWidget(i18nc("@title:window", "Information"), this);
-    m_gameDock->setObjectName("gameDock");
-    m_gameDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
+    m_gameDock->setObjectName( QLatin1String("gameDock" ));
     m_gameDock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
     GameWidget *gameWidget = new GameWidget(m_game, this);
     connect(gameWidget, SIGNAL(finishClicked()), this, SLOT(finishGame()));
     m_gameDock->setWidget(gameWidget);
-    //m_gameDock->toggleViewAction()->setText(i18nc("@title:window", "Information"));
-    //m_gameDock->toggleViewAction()->setShortcut(Qt::Key_G);
-    //actionCollection()->addAction("show_game_panel", m_gameDock->toggleViewAction());
+    m_gameDock->toggleViewAction()->setText(i18nc("@title:window", "Information"));
+    m_gameDock->toggleViewAction()->setShortcut(Qt::Key_I);
+    actionCollection()->addAction( QLatin1String( "show_game_panel" ), m_gameDock->toggleViewAction());
     addDockWidget(Qt::RightDockWidgetArea, m_gameDock);
 
     // Move history dock
     m_movesDock = new QDockWidget(i18nc("@title:window", "Moves"), this);
-    m_movesDock->setObjectName("movesDock");
+    m_movesDock->setObjectName( QLatin1String("movesDock" ));
     m_undoView = new QUndoView(m_game->undoStack());
     m_undoView->setEmptyLabel(i18n("No move"));
     m_undoView->setAlternatingRowColors(true);
@@ -466,18 +479,18 @@ void MainWindow::setupDockWindows()
     m_movesDock->setWidget(m_undoView);
     m_movesDock->toggleViewAction()->setText(i18nc("@title:window", "Moves"));
     m_movesDock->toggleViewAction()->setShortcut(Qt::Key_M);
-    actionCollection()->addAction("show_moves_panel", m_movesDock->toggleViewAction());
+    actionCollection()->addAction( QLatin1String( "show_moves_panel" ), m_movesDock->toggleViewAction());
     addDockWidget(Qt::RightDockWidgetArea, m_movesDock);
 
     m_errorDock = new QDockWidget(i18nc("@title:window", "Error"), this);
-    m_errorDock->setObjectName("errorDock");
+    m_errorDock->setObjectName( QLatin1String("errorDock" ));
     m_errorDock->setFeatures(QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
     ErrorWidget *errorWidget = new ErrorWidget(this);
     connect(errorWidget, SIGNAL(configureClicked()), this, SLOT(showPreferences()));
     m_errorDock->setWidget(errorWidget);
     //m_errorDock->toggleViewAction()->setText(i18nc("@title:window", "Error"));
     //m_errorDock->toggleViewAction()->setShortcut(Qt::Key_E);
-    //actionCollection()->addAction("show_error_panel", m_errorDock->toggleViewAction());
+    //actionCollection()->addAction( QLatin1String( "show_error_panel" ), m_errorDock->toggleViewAction());
     addDockWidget(Qt::BottomDockWidgetArea, m_errorDock);
 }
 
