@@ -83,7 +83,7 @@ BoardWidget::BoardWidget( QWidget* parent )
 }
 
 BoardWidget::~BoardWidget(){
-  if (Game) delete Game;
+  delete Game;
 }
 
 void BoardWidget::loadSettings(){
@@ -546,7 +546,7 @@ void BoardWidget::demoMoveTimeout()
                 break;
         }
         // restart timer
-        QTimer::singleShot( ANIMSPEED, this, SLOT( demoMoveTimeout() ) );
+        QTimer::singleShot( ANIMSPEED, this, SLOT(demoMoveTimeout()) );
     }
 }
 
@@ -578,7 +578,7 @@ void BoardWidget::matchAnimationTimeout()
         }
     }
     if( TimerState == Match )
-        QTimer::singleShot( ANIMSPEED, this, SLOT( matchAnimationTimeout() ) );
+        QTimer::singleShot( ANIMSPEED, this, SLOT(matchAnimationTimeout()) );
 }
 // ---------------------------------------------------------
 void BoardWidget::stopMatchAnimation()
@@ -779,10 +779,7 @@ void BoardWidget::removeTile( POSITION& Pos , bool doRepaint)
     Game->TileNum--;                    // Eine Figur weniger
     Game->setMoveListData(Game->TileNum,Pos); // Position ins Protokoll eintragen
 
-    TileSprite * thissprite =spriteMap.value(TileCoord(X,Y,E));
-    if (thissprite) delete thissprite;
-
-    spriteMap.remove(TileCoord(X,Y,E));
+    delete spriteMap.take(TileCoord(X,Y,E));
     // remove tile from game board
     Game->putTile( E, Y, X, 0 );
 }
@@ -851,9 +848,9 @@ void BoardWidget::mousePressEvent ( QMouseEvent* event )
                         gameOver(Game->MaxTileNum,cheatsUsed);
                     }
                     // else if no more moves are possible, display the sour grapes dialog
-                    else if( ! Game->findMove( TimerPos1, TimerPos2 ) )
+                    else
                     {
-                        KMessageBox::information(this, i18n("Game over: You have no moves left."));
+                      validMovesAvailable();
                     }
                 }
                 else
@@ -869,6 +866,52 @@ void BoardWidget::mousePressEvent ( QMouseEvent* event )
     }
 }
 
+//-----------------------------------------------------------
+KGameCanvasItem* BoardWidget::itemAt(const QPoint& point) const {
+    // Get the shadows...
+    // theTiles.width() == The whole tile width including offset and shadow.
+    // theTiles.qWidth() == Half of the width of the tile face (without the offset and the shadow).
+    // theTiles.levelOffsetX() == The level of the tile (the perspective height of the tile)
+    int shadowWidth = theTiles.width() - (theTiles.qWidth() * 2 + theTiles.levelOffsetX());
+    int shadowHeight = theTiles.height() - (theTiles.qHeight() * 2 + theTiles.levelOffsetY());
+
+    for (int i = m_items.size() - 1; i >= 0; i--) {
+        KGameCanvasItem *canvasItem = m_items[i];
+
+        // Cause of the shadow, that is not really a part of a tile. Correct the rect positions
+        // related to the angle. Actually we just correct the rect of the canvas item cause of the
+        // shadow, that is not part of the clickable tile.
+        QRect oldRect = canvasItem->rect();
+        QRect rectCorrection = oldRect;
+
+        // Correct the positions related to the angle we had set.
+        switch(m_angle) {
+            case NW:
+                rectCorrection.setRect(oldRect.x() + shadowWidth, oldRect.y(),
+                    oldRect.width() - shadowWidth, oldRect.height() - shadowHeight);
+                break;
+            case NE:
+                rectCorrection.setRect(oldRect.x(), oldRect.y(), oldRect.width() - shadowWidth,
+                    oldRect.height() - shadowHeight);
+                break;
+            case SW:
+                rectCorrection.setRect(oldRect.x() + shadowWidth, oldRect.y() + shadowHeight,
+                    oldRect.width() - shadowWidth, oldRect.height() - shadowHeight);
+                break;
+            case SE:
+                rectCorrection.setRect(oldRect.x(), oldRect.y() + shadowHeight,
+                    oldRect.width() - shadowWidth, oldRect.height() - shadowHeight);
+                break;
+        }
+
+        // if the canvas is visible and we click inside the corrected rect of the canvas item, we
+        // can return the actual canvas item.
+        if (canvasItem->visible() && rectCorrection.contains(point))
+            return canvasItem;
+    }
+
+    return NULL;
+}
 
 // ----------------------------------------------------------
 /*
@@ -893,11 +936,6 @@ void BoardWidget::transformPointToPosition(
     }
 
     TileCoord coord = spriteMap.key(clickedItem);
-    if (coord.isNull()) {
-	qDebug() << "null tile coordinates, clicked on background?";
-	return;
-    }
-
     E = coord.z();
     X = coord.x();
     Y = coord.y();
@@ -941,7 +979,7 @@ void BoardWidget::transformPointToPosition(
 // ---------------------------------------------------------
 bool BoardWidget::loadBoard( )
 {
-    if (Game) delete Game;
+    delete Game;
     Game = new GameData(theBoardLayout.board());
     return(true);
 }
@@ -1116,6 +1154,20 @@ void BoardWidget::shuffle() {
   // 300 points per use
   cheatsUsed += 15;
   drawTileNumber();
+
+  // Test if any moves are available
+  validMovesAvailable();
+}
+
+bool BoardWidget::validMovesAvailable()
+{
+  if(!Game->findMove(TimerPos1, TimerPos2))
+  {
+    KMessageBox::information(this, i18n("Game over: You have no moves left."));
+    return false;
+  }
+
+  return true;
 }
 
 QString  BoardWidget::getLayoutName() {
