@@ -26,6 +26,8 @@
 #include <ksavefile.h>
 
 #include <QHBoxLayout>
+#include <QLabel>
+#include <QComboBox>
 
 #include <KLocale>
 #include <KActionCollection>
@@ -114,7 +116,7 @@ void KSudoku::onCompleted(bool isCorrect, const QTime& required, bool withHelp)
 KSudoku::KSudoku()
 	: KXmlGuiWindow(), m_gameVariants(new GameVariantCollection(this, true))
 {
-	setObjectName("ksudoku");
+	setObjectName( QLatin1String("ksudoku" ));
 
 	m_gameWidget = 0;
 	m_gameUI = 0;
@@ -124,7 +126,7 @@ KSudoku::KSudoku()
 	// then, setup our actions
 	setupActions();
 
-	setupGUI(ToolBar | Keys | Save | Create);
+	setupGUI(ToolBar | Keys | Save | Create | StatusBar);
 
 	wrapper = new QWidget();
 	(void) new QHBoxLayout(wrapper);
@@ -139,6 +141,10 @@ KSudoku::KSudoku()
 	m_welcomeScreen = new WelcomeScreen(wrapper, m_gameVariants);
 	wrapper->layout()->addWidget(m_welcomeScreen);
 	connect(m_welcomeScreen, SIGNAL(newGameStarted(const ::ksudoku::Game&,GameVariant*)), this, SLOT(startGame(const ::ksudoku::Game&)));
+
+	setupStatusBar(m_welcomeScreen->difficulty(),
+		       m_welcomeScreen->symmetry());
+
 	showWelcomeScreen();
 
 	// Register the gamevariants resource
@@ -165,22 +171,10 @@ void KSudoku::updateShapesList()
 	variant = new SudokuGame(i18n("Sudoku Standard (9x9)"), 9, m_gameVariants);
 	variant->setDescription(i18n("The classic and fashionable game"));
 	variant->setIcon("ksudoku-ksudoku_9x9");
-	variant = new SudokuGame(i18n("Sudoku 16x16"), 16, m_gameVariants);
-	variant->setDescription(i18n("Sudoku with 16 symbols"));
-	variant->setIcon("ksudoku-ksudoku_16x16");
-	variant = new SudokuGame(i18n("Sudoku 25x25"), 25, m_gameVariants);
-	variant->setDescription(i18n("Sudoku with 25 symbols"));
-	variant->setIcon("ksudoku-ksudoku_25x25");
 #ifdef OPENGL_SUPPORT
 	variant = new RoxdokuGame(i18n("Roxdoku 9 (3x3x3)"), 9, m_gameVariants);
-	variant->setDescription(i18n("The Rox 3D sudoku"));
+	variant->setDescription(i18n("The Rox 3D Sudoku"));
 	variant->setIcon("ksudoku-roxdoku_3x3x3");
-	variant = new RoxdokuGame(i18n("Roxdoku 16 (4x4x4)"), 16, m_gameVariants);
-	variant->setDescription(i18n("The Rox 3D sudoku with 16 symbols"));
-	variant->setIcon("ksudoku-roxdoku_4x4x4");
-	variant = new RoxdokuGame(i18n("Roxdoku 25 (5x5x5)"), 25, m_gameVariants);
-	variant->setDescription(i18n("The Rox 3D sudoku with 25 symbols"));
-	variant->setIcon("ksudoku-roxdoku_5x5x5");
 #endif
 
     QStringList filepaths = KGlobal::dirs()->findAllResources("gamevariant", "*.desktop", KStandardDirs::NoDuplicates); // Find files.
@@ -199,12 +193,29 @@ void KSudoku::updateShapesList()
 		variantIcon = group.readEntry("Icon", "ksudoku-ksudoku_9x9");
 		variantDataPath = group.readEntry("FileName", "");
 		if(variantDataPath == "") continue;
+
 		variantDataPath = filepath.left(filepath.lastIndexOf("/")+1) + variantDataPath;
 
 		variant = new CustomGame(variantName, variantDataPath, m_gameVariants);
 		variant->setDescription(variantDescr);
 		variant->setIcon(variantIcon);
 	}
+
+	// Put variants first and extra sizes last.
+	variant = new SudokuGame(i18n("Sudoku 16x16"), 16, m_gameVariants);
+	variant->setDescription(i18n("Sudoku with 16 symbols"));
+	variant->setIcon("ksudoku-ksudoku_16x16");
+	variant = new SudokuGame(i18n("Sudoku 25x25"), 25, m_gameVariants);
+	variant->setDescription(i18n("Sudoku with 25 symbols"));
+	variant->setIcon("ksudoku-ksudoku_25x25");
+#ifdef OPENGL_SUPPORT
+	variant = new RoxdokuGame(i18n("Roxdoku 16 (4x4x4)"), 16, m_gameVariants);
+	variant->setDescription(i18n("The Rox 3D sudoku with 16 symbols"));
+	variant->setIcon("ksudoku-roxdoku_4x4x4");
+	variant = new RoxdokuGame(i18n("Roxdoku 25 (5x5x5)"), 25, m_gameVariants);
+	variant->setDescription(i18n("The Rox 3D sudoku with 25 symbols"));
+	variant->setIcon("ksudoku-roxdoku_5x5x5");
+#endif
 }
 
 void KSudoku::startGame(const Game& game) {
@@ -289,6 +300,9 @@ void KSudoku::autoSolve()
 	game.autoSolve();
 }
 
+// Check the game setup, copy the puzzle, init and solve the copy and show the
+// result (i.e. implement the "Check" action). If the user agrees, start play.
+
 void KSudoku::dubPuzzle()
 {
 	Game game = currentGame();
@@ -300,7 +314,11 @@ void KSudoku::dubPuzzle()
 		return;
 	}
 
+	// Create a new Puzzle object, with same Graph and solution flag = true.
 	ksudoku::Puzzle* puzzle = game.puzzle()->dubPuzzle();
+
+	// Copy the given values of the puzzle, then run it through the solver.
+	// The solution, if valid, is saved in puzzle->m_solution2.
 	int state = puzzle->init(game.allValues());
 
 	if(state <= 0) {
@@ -359,20 +377,71 @@ void KSudoku::setupActions()
 	KStandardGameAction::solve(this, SLOT(autoSolve()), actionCollection());
 
 	KAction* a = new KAction(this);
-	actionCollection()->addAction("move_dub_puzzle", a);
+	actionCollection()->addAction( QLatin1String( "move_dub_puzzle" ), a);
 	a->setText(i18n("Check"));
-	a->setIcon(KIcon("games-endturn"));
+	a->setIcon(KIcon( QLatin1String( "games-endturn" )));
 	connect(a, SIGNAL(triggered(bool)), SLOT(dubPuzzle()));
 	addAction(a);
 
 	//WEB
 	a = new KAction(this);
-	actionCollection()->addAction("home_page", a);
+	actionCollection()->addAction( QLatin1String( "home_page" ), a);
 	a->setText(i18n("Home Page"));
-	a->setIcon(KIcon("internet-web-browser"));
+	a->setIcon(KIcon( QLatin1String( "internet-web-browser" )));
 	connect(a, SIGNAL(triggered(bool)), SLOT(homepage()));
 }
 
+void KSudoku::setupStatusBar (int difficulty, int symmetry)
+{
+	// Use the standard combo box for difficulty, from KDE Games library.
+	const int nStandardLevels = 4;
+	const KGameDifficulty::standardLevel standardLevels[nStandardLevels] =
+			{KGameDifficulty::VeryEasy, KGameDifficulty::Easy,
+			 KGameDifficulty::Medium,   KGameDifficulty::Hard};
+
+	statusBar()->addPermanentWidget (new QLabel (i18n("Difficulty")));
+	KGameDifficulty::init (this, this,
+		SLOT (difficultyChanged(KGameDifficulty::standardLevel)),
+		SLOT (difficultyChanged(int)));
+	KGameDifficulty::addStandardLevel(KGameDifficulty::VeryEasy);
+	KGameDifficulty::addStandardLevel(KGameDifficulty::Easy);
+	KGameDifficulty::addStandardLevel(KGameDifficulty::Medium);
+	KGameDifficulty::addStandardLevel(KGameDifficulty::Hard);
+	KGameDifficulty::addCustomLevel(Diabolical,
+		i18nc("A level of difficulty in Sudoku puzzles", "Diabolical"));
+	KGameDifficulty::addCustomLevel(Unlimited,
+		i18nc("A level of difficulty in Sudoku puzzles", "Unlimited"));
+	KGameDifficulty::setRestartOnChange(KGameDifficulty::NoRestartOnChange);
+
+	// Set default value of difficulty.
+	if (difficulty < nStandardLevels) {
+	    KGameDifficulty::setLevel (standardLevels[difficulty]);
+	}
+	else {
+	    KGameDifficulty::setLevelCustom (difficulty);
+	}
+	KGameDifficulty::setEnabled (true);
+
+	// Set up a combo box for symmetry of puzzle layout.
+	statusBar()->addPermanentWidget (new QLabel (i18n("Symmetry")));
+	QComboBox * symmetryBox = new QComboBox (this);
+	QObject::connect(symmetryBox, SIGNAL(activated(int)),
+		    		this, SLOT(symmetryChanged(int)));
+	symmetryBox->setToolTip(i18nc(
+		"Symmetry of layout of clues when puzzle starts", "Symmetry"));
+	symmetryBox->setWhatsThis(i18n(
+		"The symmetry of layout of the clues when the puzzle starts"));
+	statusBar()->addPermanentWidget (symmetryBox);
+	symmetryBox->addItem(i18nc("Symmetry of layout of clues", "Diagonal"));
+	symmetryBox->addItem(i18nc("Symmetry of layout of clues", "Central"));
+	symmetryBox->addItem(i18nc("Symmetry of layout of clues", "Left-Right"));
+	symmetryBox->addItem(i18nc("Symmetry of layout of clues", "Spiral"));
+	symmetryBox->addItem(i18nc("Symmetry of layout of clues", "Four-Way"));
+	symmetryBox->addItem(i18nc("Symmetry of layout of clues", "Random Choice"));
+	symmetryBox->addItem(i18n("No Symmetry"));
+	symmetryBox->setCurrentIndex (symmetry);
+}
+
 void KSudoku::adaptActions2View() {
 	Game game = currentGame();
 
@@ -599,6 +668,51 @@ void KSudoku::updateSettings() {
 	emit settingsChanged();
 }
 
+void KSudoku::difficultyChanged (KGameDifficulty::standardLevel difficulty)
+{
+    qDebug() << "Set difficulty =" << difficulty;
+    int newDifficulty = VeryEasy;
+    switch (difficulty) {
+    case KGameDifficulty::VeryEasy:
+	newDifficulty = VeryEasy;
+	break;
+    case KGameDifficulty::Easy:
+	newDifficulty = Easy;
+	break;
+    case KGameDifficulty::Medium:
+	newDifficulty = Medium;
+	break;
+    case KGameDifficulty::Hard:
+	newDifficulty = Hard;
+	break;
+    default:
+	return;
+    }
+    qDebug() << "Set new difficulty =" << newDifficulty;
+    m_welcomeScreen->setDifficulty(newDifficulty);
+    return;
+}
+
+void KSudoku::difficultyChanged (int difficulty)
+{
+    qDebug() << "Set custom difficulty =" << difficulty;
+    m_welcomeScreen->setDifficulty(difficulty);
+    if (difficulty == Unlimited) {
+	KMessageBox::information (this,
+		i18n("Warning: The Unlimited difficulty level has no limit on "
+		     "how many guesses or branch points are required to solve "
+		     "the puzzle and there is no lower limit on how soon "
+		     "guessing becomes necessary."),
+		i18n("Warning"), "WarningReUnlimited");
+    }
+}
+
+void KSudoku::symmetryChanged (int symmetry)
+{
+    qDebug() << "Set symmetry =" << symmetry;
+    m_welcomeScreen->setSymmetry(symmetry);
+}
+
 // void KSudoku::changeStatusbar(const QString& text)
 // {
 //     // display the text on the statusbar
